////////////////////////////////////////////////////////
// DLL and unix - Dynamic link
//
// Sylvain Marechal
////////////////////////////////////////////////////////
#include <stdio.h>
#include <dlfcn.h>

typedef void (*PFHelloShared)(int );

int main(void) 
{
    const char *error;
    void *module;
    PFHelloShared pfHelloShared;
    
    /* Load dynamically loaded library */
    module = dlopen("./libshared.so", RTLD_LAZY);
    if (!module) 
    {
	fprintf(stderr, "Couldn't open the .so: %s\n",
	    dlerror());
	exit(1);
    }
    
    /* Get symbol */
    dlerror();
    pfHelloShared = (PFHelloShared)dlsym(module, "HelloShared");
    if ((error = dlerror())) 
    {
	fprintf(stderr, "Couldn't find hello: %s\n", error);
	exit(1);
    }
    
    /* Now call the function in the DL library */
    (*pfHelloShared)(8888);
    
    /* All done, close things cleanly */
    dlclose(module);
    return 0;
}
